import os
import joblib
import pytest
import numpy as np
from unittest.mock import patch
from dodfminer.extract.polished.backend.ner import ActNER

@pytest.fixture
@patch.object(ActNER, '_load_model',
              return_value=joblib.load(""+os.path.dirname(__file__)+"/support/test_model.pkl"))

def act_ner_with_model(mock_model):
    act = ActNER()
    act._name = ""
    act._backend = 'ner'
    return act

def test_act_ner_backend_ner(act_ner_with_model):
    assert act_ner_with_model._backend == 'ner'

def test_act_ner_preprocess(act_ner_with_model):
    # Caso 1 (Sentenca inventada)
    str1 = "hoje, eu vou. amanha, talvez. pode ser\n que sim; pode\n ser que nao"
    str2 = ["hoje", ",", "eu", "vou", ".", "amanha", ",", "talvez", ".", "pode", "ser", "que", "sim", ";", "pode", "ser", "que", "nao"]
    test1= (act_ner_with_model._preprocess(str1)==str2)
    # Caso 2 (Sentenca de um DODF)
    str1 = "Art. 4o Validar os atos escolares praticados pela instituicao educacional, a contar de 1o de\njaneiro de 2012 ate a data de publicacao da portaria oriunda do presente parec e r. "
    str2 = ["Art", ".", "4o", "Validar", "os", "atos", "escolares", "praticados", "pela", "instituicao", "educacional", ",", "a", "contar", "de", "1o", "de", "janeiro", "de", "2012", "ate", "a", "data", "de", "publicacao", "da", "portaria", "oriunda", "do", "presente", "parec", "e", "r", "."]
    test2= (act_ner_with_model._preprocess(str1)==str2)
    assert test1 and test2

def test_act_ner_get_features(act_ner_with_model):

    list_1 = ["Art", ".", "4", "Validar", "ATOS", "."]
    features_1 = [{'word': 'art',
                'capital_letter': True,
                'all_capital': False,
                'isdigit': False,
                'word_before': 'art',
                'word_after:': '.',
                'BOS': True,
                'EOS': False
                },
                {'word': '.',
                'capital_letter': False,
                'all_capital': False,
                'isdigit': False,
                'word_before': 'art',
                'word_after:': '4',
                'BOS': False,
                'EOS': False
                },
                {'word': '4',
                'capital_letter': False,
                'all_capital': False,
                'isdigit': True,
                'word_before': '.',
                'word_after:': 'validar',
                'BOS': False,
                'EOS': False
                },
                {'word': 'validar',
                'capital_letter': True,
                'all_capital': False,
                'isdigit': False,
                'word_before': '4',
                'word_after:': 'atos',
                'BOS': False,
                'EOS': False
                },
                {'word': 'atos',
                'capital_letter': True,
                'all_capital': True,
                'isdigit': False,
                'word_before': 'validar',
                'word_after:': '.',
                'BOS': False,
                'EOS': False
                },
                {'word': '.',
                'capital_letter': False,
                'all_capital': False,
                'isdigit': False,
                'word_before': 'atos',
                'word_after:': '.',
                'BOS': False,
                'EOS': True
                }]
    assert features_1 == act_ner_with_model._get_features(list_1)

def test_act_ner_prediction(act_ner_with_model):
    sentence    = "matricula 190133743 joao souza artigo 5o data 03/12/1901 cargo professor"
    # Step-by-step execution of _prediction()
    sent = act_ner_with_model._preprocess(sentence)
    feats = act_ner_with_model._get_features(sent)
    predictions = act_ner_with_model._model.predict_single(feats)
    entities_predicted_StepByStep = act_ner_with_model._predictions_dict(sent, predictions)
    # Direct execution of _prediction()
    entities_predicted_at_once = act_ner_with_model._prediction(sentence)

    # Evaluating _model.predict_single()
    test1 = True
    for prediction in predictions:
        if prediction not in act_ner_with_model._model.classes_:
            test1 = False
    # Check if step-by-step is equivalent to at_once execution
    test2 = entities_predicted_at_once == entities_predicted_StepByStep
    assert test1 and test2

def test_act_ner_predictions_dict(act_ner_with_model):
    prediction = ["B-matricula", "I-matricula", "B-Nome do Servidor", "I-Nome do Servidor", "B-fundamento legal do abono de permanencia", "I-fundamento legal do abono de permanencia", "B-Vigencia", "I-Vigencia", "B-Cargo Efetivo", "I-Cargo Efetivo"]
    sentence    = ["matricula"  , "190133743"  , "joao"              , "souza"             , "artigo"                                    , "5o"                                        ,  "data"     , "03/12/1901", "cargo"          , "professor"]
    entidades = {
        'Tipo do Ato': '',
        'matricula': "matricula 190133743",
        'Nome do Servidor': "joao souza",
        'fundamento legal do abono de permanencia': "artigo 5o",
        'Vigencia': "data 03/12/1901",
        'Cargo Efetivo': "cargo professor",
        'Classe': np.nan,
        'padrao': np.nan,
        'Quadro pessoal permanente ou Suplementar': np.nan,
        'Matricula SIAPE': np.nan,
        'Processo GDF/SEI': np.nan,
        'orgao': np.nan
    }
    assert entidades == act_ner_with_model._predictions_dict(sentence, prediction)



try:

    from dodfminer.extract.polished import acts
    from dodfminer.downloader.core import Downloader
    from dodfminer.extract.polished.core import ActsExtractor
    from dodfminer.extract.pure.core import ContentExtractor
except:
    pass


#!/usr/bin/env python3

"""Main application module.

Contains class miner which is an interface to handle all extraction tasks.

Usage example::

    dodfminer --help

"""

from dodfminer.cli import CLI
from dodfminer.downloader.core import Downloader
from dodfminer.extract.pure.core import ContentExtractor
from dodfminer.extract.polished.helper import extract_multiple_acts, xml_multiple


class Miner(object):
    """Main DODFMiner class.

    The Miner class is a interface that handles download, pre-extraction,
    and extraction of DODFs.

    Attributes:
        start_date: Argument start_date from CLI
        end_date: Argument end_date from CLI
        single: Argument single from CLI
        ext_content: Argument extract_content from CLI

    """

    def __init__(self):
        """Init Miner class to handle all the extraction process."""
        self.cli = CLI()
        self.args = self.cli.parse()

    def download(self):
        """Download PDFs with parameters from CLI."""
        downloader = Downloader(save_path=self.args.save_path)
        downloader.pull(self.args.start_date, self.args.end_date)

    def extract_content(self):
        """Extract Content from PDFs."""
        if self.args.single_file is None:
            if self.args.type_of_extr is not None:
                if self.args.type_of_extr == 'pure-text':
                    ContentExtractor.extract_to_txt(folder=self.args.input_folder)
                elif self.args.type_of_extr == 'with-titles':
                    ContentExtractor.extract_to_json(folder=self.args.input_folder,
                                                     titles_with_boxes=True)
                elif self.args.type_of_extr == 'blocks':
                    ContentExtractor.extract_to_json(folder=self.args.input_folder)
            elif self.args.act != 'all':
                extract_multiple_acts(self.args.input_folder, self.args.act, self.args.backend)
            elif self.args.xml != False:
                xml_multiple(self.args.input_folder, self.args.backend)
            else:
                self.cli.extract_content_parser.print_help()
        elif self.args.single_file is not None:
            if self.args.type_of_extr is not None:
                if self.args.type_of_extr == 'pure-text':
                    ContentExtractor.extract_text(self.args.single_file,
                                                  single=True)
                elif self.args.type_of_extr == 'with-titles':
                    ContentExtractor.extract_structure(self.args.single_file,
                                                       single=True)
                elif self.args.type_of_extr == 'blocks':
                    ContentExtractor.extract_text(self.args.single_file, single=True, block=True)
            elif self.args.act != 'all':
                extract_multiple_acts(self.args.single_file, self.args.act, self.args.backend)
            elif self.args.xml != False:
                xml_multiple(self.args.single_file, self.args.backend)
            else:
                self.cli.extract_content_parser.print_help()

    def _log(self, msg):
        print(f"[DODFMiner] {msg}")


def run():
    miner = Miner()
    if miner.args.subparser_name == 'downloader':
        miner.download()
    elif miner.args.subparser_name == 'extract':
        miner.extract_content()
    else:
        miner.cli.parser.print_help()

if __name__ == '__main__':
    run()



import os
import re
import pytest
import joblib
import sklearn_crfsuite
from unittest.mock import patch

from dodfminer.extract.polished.acts.aposentadoria import Retirements, RetAposentadoria
from dodfminer.extract.polished.acts.nomeacao import NomeacaoComissionados, NomeacaoEfetivos
from dodfminer.extract.polished.acts.exoneracao import Exoneracao, ExoneracaoEfetivos
from dodfminer.extract.polished.acts.reversoes import Revertions
from dodfminer.extract.polished.acts.abono import AbonoPermanencia
from dodfminer.extract.polished.acts.substituicao import Substituicao
from dodfminer.extract.polished.acts.cessoes import Cessoes
from dodfminer.extract.polished.acts.sem_efeito_aposentadoria import SemEfeitoAposentadoria

file = ""+os.path.dirname(__file__)+"/support/valid.txt"

@pytest.fixture()
def act_ret():
    return Retirements(file, 'regex')

def test_retirement_backend(act_ret):
    assert act_ret._backend == "regex"

def test_retirement_name(act_ret):
    assert act_ret._name == "Aposentadoria"
    assert act_ret.name == "Aposentadoria"
    assert act_ret._act_name() == "Aposentadoria"

def test_retirement_flags(act_ret):
    assert act_ret._regex_flags() == re.IGNORECASE

def test_retirement_prop_names(act_ret):
    assert act_ret._props_names() == ["Tipo do Ato", "SEI", "Nome", "Matrícula", "Tipo de Aposentadoria", "Cargo", "Classe", "Padrao", "Quadro", "Fundamento Legal", "Orgao", "Vigencia", "Matricula SIAPE"]

def test_retirement_rule(act_ret):
    assert act_ret._rule_for_inst() == r"(APOSENTAR|CONCEDER,\sAPOSENTADORIA|CONCEDER\sAPOSENTADORIA,?\s?)([\s\S]*?(?<!lei)\s(?:[0-9|\s]*?[.|-]\s?)+?[0-9|\s]*/\s?[0-9|\s]*-?\s?[0-9|\s]*[.|,])"

def test_retirement_prop_rules_names(act_ret):
    assert list(act_ret._prop_rules()) == ["processo_SEI", "nome", "matricula", "tipo_ret", "cargo_efetivo", "classe", "padrao", "quadro", "fundamento_legal", "orgao", "vigencia", "matricula_SIAPE"]

def test_retirement_prop_rules_rules(act_ret):
    assert list(act_ret._prop_rules().values()) == [
        r"(?<!lei)\s((?:[0-9|\s]*?[.|-]\s?)+?[0-9|\s]*/\s?[0-9|\s]*-?\s?[0-9|\s]*)[.|,]",
        r"\s([^,]*?),\smatricula",
        r"matricula\s?n?o?\s([\s\S]*?)[,|\s]",
        r"",
        r"Cargo de([\s\S]*?)\,",
        r"[C|c]lasse\s([\s\S]*?)\,",
        r"[p|P]adr[a|ã]o\s([\s\S]*?),",
        r"d?[e|a|o]?(Quadro[\s\S]*?)[,|;|.]",
        r"nos\stermos\sdo\s[a|A]rtigo([\s\S]*?),\sa?\s",
        r"Lotacao:|Quadro\sde\sPessoal\sd[a|e|o]([\s\S]*?)[.|,]",
        r"",
        r"[S|s][I|i][A|a][P|p][E|e]\s[N|n]?[o|O]?\s([\s\S]*?)[,| | .]"
    ]

def test_act_retirement_consistence_rule(act_ret):
    assert len(act_ret._props_names())-1 == len(act_ret._prop_rules())

def test_act_retirement_ner():
    act = Retirements(file, 'ner')
    assert type(act._load_model()) == sklearn_crfsuite.estimator.CRF

#
#
#
#
#

@pytest.fixture()
def act_retapos():
    return RetAposentadoria(file, 'regex')

def test_retretirement_backend(act_retapos):
    assert act_retapos._backend == "regex"

def test_retretirement_name(act_retapos):
    assert act_retapos._name == "Retificações de Aposentadoria"
    assert act_retapos.name == "Retificações de Aposentadoria"
    assert act_retapos._act_name() == "Retificações de Aposentadoria"

def test_retretirement_flags(act_retapos):
    assert act_retapos._regex_flags() == re.IGNORECASE

def test_retretirement_prop_names(act_retapos):
    assert act_retapos._props_names() == ["Tipo do Ato", "Tipo de Documento", "Número do documento",
                "Data do documento ", "Número do DODF", "Data do DODF",
                "Página do DODF", "Nome do Servidor", "Matrícula", "Cargo",
                "Classe", "Padrao", "Matricula SIAPE",
                "Informação Errada", "Informação Corrigida"]

def test_retretirement_rule(act_retapos):
    assert act_retapos._rule_for_inst() == r"(RETIFICAR,\s)(.*?ato\sque\sconcedeu\saposentadoria[\s\S]*?\.\n)"

def test_retretirement_prop_rules_names(act_retapos):
    assert list(act_retapos._prop_rules()) == [
        "tipo_documento", "numero_documento", "data_documento", "numero_dodf", "data_dodf", "pagina_dodf", "nome", "matricula", "cargo_efetivo","classe", "padrao", "matricula_SIAPE","informacao_errada", "informacao_corrigida"]

def test_retretirement_prop_rules_rules(act_retapos):
    assert list(act_retapos._prop_rules().values()) == [
        r"^n[a|o]\s([\s\S]*?),?\s?(?:[0-9]*?),?\sde\s(?:[0-9]*?[/|.][0-9]*?[/|.][0-9]*?|,)",
        r"n[a|o]\s(?:[\s\S]*?),?\s?([0-9]*?),?\sde\s(?:[0-9]*?[/|.][0-9]*?[/|.][0-9]*?|,)",
        r"n[a|o]\s(?:[\s\S]*?),?\s?(?:[0-9]*?),?\sde\s([0-9]*?[/|.][0-9]*?[/|.][0-9]*?),\s",
        r"dodf[\s\S]*?([0-9]*?),",
        r"dodf[\s\S]*?(?:[0-9]*?)([0-9]*?[/|.][0-9]*?[/|.][0-9]*?)[,|\s]",
        r"",
        r"\sa\s([^,]*?),\smatricula",
        r"matricula\s?n?o?\s([\s\S]*?-[\s\S]*?)[,]",
        r"(?:Cargo|Carreira)\sde([\s\S]*?)\,",
        r"(?:([^,]*?)\sclasse,)?(?(1)|classe\s([\s\S]*?),)",
        r"[p|P]adr[a|ã]o\s([\s\S]*?),",
        r"siape\sn?o?\s([\s\S]*?)[,| | .]",
        r"\sle[,|:|;]\s?([\s\S]*?),?\sleia[\s\S]*?[,|:|;]\s(?:[\s\S]*?)[.]\s",
        r"\sle[,|:|;]\s?(?:[\s\S]*?),?\sleia[\s\S]*?[,|:|;]\s([\s\S]*?)[.]\s"
    ]

def test_act_retretirement_consistence_rule(act_retapos):
    assert len(act_retapos._props_names())-1 == len(act_retapos._prop_rules())

def test_act_retretirement_ner():
    act = RetAposentadoria(file, 'ner')
    assert act._load_model() == None

#
#
#
#
#

@pytest.fixture()
def act_abono():
    return AbonoPermanencia(file, 'regex')

def test_abono_backend(act_abono):
    assert act_abono._backend == "regex"

def test_abono_name(act_abono):
    assert act_abono._name == "Abono de Permanência"
    assert act_abono.name == "Abono de Permanência"
    assert act_abono._act_name() == "Abono de Permanência"

def test_abono_flags(act_abono):
    assert act_abono._regex_flags() == re.IGNORECASE

def test_abono_prop_names(act_abono):
    assert act_abono._props_names() == ["Tipo do Ato", "Nome do Servidor", "Matrícula",
                "Cargo Efetivo", "Classe", "Padrão",
                "Quadro pessoal permanente ou Suplementar",
                "Fundamento Legal do abono de permanência", "Órgão",
                "Processo GDF/SEI", "Vigencia", "Matricula SIAPE"]

def test_abono_rule(act_abono):
    assert act_abono._rule_for_inst() == r"(Abono\sDE\sPERMANENCIA\s[(ao|equiva)][\s\S]*?)\s([\s\S]*?\.\n)"

def test_abono_prop_rules_names(act_abono):
    assert list(act_abono._prop_rules()) == ["nome",  "matricula", "cargo_efetivo", "classe", "padrao",  "quadro", "fundamento_legal_abono_permanencia", "orgao",
    "processo_SEI", "vigencia", "matricula_SIAPE"]

def test_abono_prop_rules_rules(act_abono):
    assert list(act_abono._prop_rules().values()) == [
        r"\s([^,]*?),\smatricula",
        r"matricula\s?n?o?\s([\s\S]*?)[,|\s]",
        r"Cargo\s[d|D]?[e|E]?\s([\s\S]*?),",
        r"[C|c]lasse\s([\s\S]*?),",
        r"[p|P]adr[a|ã]o\s([\s\S]*?),",
        r"d?[e|a|o]?(Quadro[\s\S]*?)[,|;|.]",
        r"nos\stermos\sdo\s([\s\S]*?),\sa?\s",
        r"Lotacao: ([\s\S]*?)[.]",
        r"Processo SEI: ([\s\S]*?)\.\n",
        r"a contar de ([\s\S]*?)\,",
        r"[S|s][I|i][A|a][P|p][E|e]\s[N|n]?[o|O]?\s([\s\S]*?)[,| | .]"
    ]


def test_act_abono_consistence_rule(act_abono):
    assert len(act_abono._props_names())-1 == len(act_abono._prop_rules())

def test_act_abono_ner():
    act = AbonoPermanencia(file, 'ner')
    assert type(act._load_model()) == sklearn_crfsuite.estimator.CRF

#
#
#
#
#


@pytest.fixture()
def act_exoneracao():
    return Exoneracao(file, 'regex')

def test_exoneracao_backend(act_exoneracao):
    assert act_exoneracao._backend == "regex"

def test_exoneracao_name(act_exoneracao):
    assert act_exoneracao._name == "Exoneração"
    assert act_exoneracao.name == "Exoneração"
    assert act_exoneracao._act_name() == "Exoneração"

def test_exoneracao_flags(act_exoneracao):
    assert act_exoneracao._regex_flags() == 0

def test_exoneracao_prop_names(act_exoneracao):
    assert act_exoneracao._props_names() == ['Tipo do Ato', 'nome', 'matricula', 'simbolo', 'cargo_comissao',
                'lotacao', 'orgao', 'vigencia', 'pedido', 'cargo_efetivo',
                'siape', 'motivo']

def test_exoneracao_rule(act_exoneracao):
    assert act_exoneracao._rule_for_inst() == r"(EXONERAR)([\s\S]*?)\."

def test_exoneracao_prop_rules_names(act_exoneracao):
    assert list(act_exoneracao._prop_rules()) == [
              "nome",
         "matricula",
           "simbolo",
"cargo_comissionado",
 "hierarqui_lotacao",
             "orgao",
          "vigencia",
   "a_pedido_ou_nao",
     "cargo_efetivo",
   "matricula_SIAPE",
            "motivo"
    ]

def test_exoneracao_prop_rules_rules(act_exoneracao):
    assert list(act_exoneracao._prop_rules().values()) == [
        r"([A-ZÀ-Ž\s]+[A-ZÀ-Ž])",
        r"matricula\s?n?o?\s([\s\S]*?-[\s\S]*?)[,]",
        r"[S|s][í|i]mbolo\s?n?o?\s([\s\S]*?)[,|\s]",
        r"(?:Cargo|Carreira)\sde([\s\S]*?)\,",
        r"(?:[S|s][í|i]mbolo\s?n?o?\s(?:[\s\S]*?)[,|\s])\sde(?:[\s\S]*?),\sd[a|e|o]\s([\s\S]*,?),",
        r"(?:[S|s][í|i]mbolo\s?n?o?\s(?:[\s\S]*?)[,|\s])\sde(?:[\s\S]*?),\sd[a|e|o]\s(?:[\s\S]*,?),\sd[a|e|o]\s([\s\S]*?)$",
        r"",
        r"(a pedido)",
        r"",
        r"[S|s][I|i][A|a][P|p][E|e]\s[N|n]*[o|O]*\s?([\s\S]*?)[,| | .]",
        r""
    ]

def test_act_exoneracao_consistence_rule(act_exoneracao):
    assert len(act_exoneracao._props_names())-1 == len(act_exoneracao._prop_rules())

def test_act_exo_name_ner():
    act = Exoneracao(file, 'ner')
    assert act._load_model() == None

#
#
#
#
#

@pytest.fixture()
def act_exo_efet():
    return ExoneracaoEfetivos(file, 'regex')

def test_exo_efet_backend(act_exo_efet):
    assert act_exo_efet._backend == "regex"

def test_exo_efet_name(act_exo_efet):
    assert act_exo_efet._name == "Exoneração Efetivos"
    assert act_exo_efet.name == "Exoneração Efetivos"
    assert act_exo_efet._act_name() == "Exoneração Efetivos"

def test_exo_efet_flags(act_exo_efet):
    assert act_exo_efet._regex_flags() == 0

def test_exo_efet_prop_names(act_exo_efet):
    assert act_exo_efet._props_names() == ['tipo', 'nome','matricula','cargo_efetivo','classe','padrao','carreira','quadro', 'SEI','data','pedido', 'motivo', 'SIAPE', 'fundalamento_legal']

def test_exo_efet_rule(act_exo_efet):
    assert act_exo_efet._rule_for_inst() == r"(EXONERAR,\s?)((?:a\spedido,)?\s(?:[A-Z\\n\s]+),\s(?:matr[ií]cula\s(?:[0-9\.-])+)[,\sa-zA-Z0-9\\\/-]*)"

def test_exo_efet_prop_rules_names(act_exo_efet):
    assert list(act_exo_efet._prop_rules()) == [
            "nome",
       "matricula",
   "cargo_efetivo",
          "classe",
          "padrao",
        "carreira",
          "quadro",
    "processo_SEI",
            "data",
          "pedido",
          "motivo",
 "matricula_SIAPE",
"fundamento_legal"
    ]

def test_exo_efet_prop_rules_rules(act_exo_efet):
    assert list(act_exo_efet._prop_rules().values()) == [
        r"^(?:a\spedido,)?\s([A-Z\\n\s]+)",
        r"matr[í|i]cula\s?n?o?\s([\s\S]*?)[,|\s]",
        r"matr[í|i]cula\s?n?o?\s[0-9]+,?([\sa-zA-Z]+)",
        r"matr[í|i]cula\s?n?o?\s[0-9]+,?[\sa-zA-Z]+[,\\n\s]+[eE]specialidade\s?([\sa-zA-Z]+)a\s?contar",
        r"",
        r"",
        r"",
        r"SEI[a-z\s]*([0-9\-\/\n]+)",
        r"a\scontar\sde\s([\s0-9\/]*)",
        r"(a\spedido,)?\s(?:[A-Z\\n\s]+)",
        r"",
        r"",
        r"nos\stermos\sdo[\n]?([a-zA-Z\s0-9\/]*)"
    ]

def test_act_exo_efet_consistence_rule(act_exo_efet):
    assert len(act_exo_efet._props_names())-1 == len(act_exo_efet._prop_rules())


def test_act_exo_efet_ner():
    act = ExoneracaoEfetivos(file, 'ner')
    assert act._load_model() == None


#
#
#
#
#

@pytest.fixture()
def act_subs():
    return Substituicao(file, 'regex')

def test_substituicao_backend(act_subs):
    assert act_subs._backend == "regex"

def test_substituicao_name(act_subs):
    assert act_subs._name == "Substituição de Funções"
    assert act_subs.name == "Substituição de Funções"
    assert act_subs._act_name() == "Substituição de Funções"

def test_substituicao_flags(act_subs):
    assert act_subs._regex_flags() == re.IGNORECASE

def test_substituicao_prop_names(act_subs):
    assert act_subs._props_names() == ["Tipo do Ato", "Nome do Servidor Substituto",
                "Matrícula do Servidor Substituto",
                "Nome do Servidor a ser Substituido",
                "Matrícula do Servidor a ser Substituido"
                "Cargo", "Símbolo do cargo do servidor substituto",
                "Cargo comissionado objeto da substituição",
                "Símbolo do cargo do objeto da substituição",
                "Símbolo do cargo comissionado objeto da substituição",
                "Hierarquia da Lotação", "Órgão", "Data Inicial da Vigência",
                "Data Final de Vigência", "Matrícula SIAPE", "Motivo"]

def test_substituicao_rule(act_subs):
    assert act_subs._rule_for_inst() == r"(DESIGNAR)([\s\S]*?)\.\s"

def test_substituicao_prop_rules_names(act_subs):
    assert list(act_subs._prop_rules()) == [
    "nome_substituto",
    "matricula_substituto",
    "nome_substituido",
    "matricula_substituido",
    "cargo_substituto",
    "simbolo_substituto",
    "cargo_objeto_substituicao",
    "simbolo_objeto_substituicao",
    "hierarquia_lotacao",
    "orgao",
    "data_inicial",
    "data_final",
    "matricula_SIAPE",
    "motivo"
    ]

def test_substituicao_prop_rules_rules(act_subs):
    assert list(act_subs._prop_rules().values()) == [
        r"(^[A-ZÀ-Ž\s]+[A-ZÀ-Ž])",
        r"(?:^[A-ZÀ-Ž\s]+[A-ZÀ-Ž])\s[\s\S]*?\smatr[í|i]cula\s?n?o?\s([\s\S]*?)[,|\s]",
        r"para\ssubstituir\s([A-ZÀ-Ž\s]+[A-ZÀ-Ž])",
        r"matr[í|i]cula\s?n?o?\s(?:[\s\S]*?)[\s\S]*?matr[í|i]cula\s?n?o?\s([\s\S]*?),",
        r"para\ssubstituir\s(?:[A-ZÀ-Ž\s]+[A-ZÀ-Ž]),\smatr[í|i]cula\s?n?o?\s(?:[\s\S]*?),\s([\s\S]*?),",
        r"[S|s][í|i]mbolo\s?n?o?\s([\s\S]*?)[,|\s]",
        "",
        r"[S|s][í|i]mbolo\s?n?o?\s(?:[\s\S]*?)[\s\S]*?[S|s][í|i]mbolo\s?n?o?\s([\s\S]*?)",
        "",
        r"(?:[S|s][í|i]mbolo\s?n?o?\s(?:[\s\S]*?)[,|\s])\sde(?:[\s\S]*?),\sd[a|e|o]\s(?:[\s\S]*,?),\sd[a|e|o]\s([\s\S]*?)$",
        "",
        "",
        r"[S|s][I|i][A|a][P|p][E|e]\s[N|n]?[o|O]?\s([\s\S]*?)[,| | .]",
        ""
    ]

def test_act_substitution_consistence_rule(act_subs):
    assert len(act_subs._props_names())-1 == len(act_subs._prop_rules())

def test_act_substituicao_ner():
    act = Substituicao(file, 'ner')
    assert act._load_model() == None

#
#
#
#
#

@pytest.fixture()
def act_revert():
    return Revertions(file, 'regex')

def test_revertions_backend(act_revert):
    assert act_revert._backend == "regex"

def test_revertions_name(act_revert):
    assert act_revert._name == "Reversão"
    assert act_revert.name == "Reversão"
    assert act_revert._act_name() == "Reversão"

def test_revertions_flags(act_revert):
    assert act_revert._regex_flags() == re.IGNORECASE

def test_revertions_prop_names(act_revert):
    assert act_revert._props_names() == ["Tipo do Ato", "SEI", "Nome", "Matricula", "Cargo", "Classe",
                "Padrao", "Quadro", "Fundamento Legal", "Orgao",
                "Vigencia", "Matricula SIAPE"]

def test_revertions_rule(act_revert):
    assert act_revert._rule_for_inst() == r"(reverter\sa\satividade[,|\s])([\s\S]*?(?<!lei)\s(?:[0-9|\s]*?[.|-]\s?)+?[0-9|\s]*/\s?[0-9|\s]*-?\s?[0-9|\s]*[.|,])"

def test_revertions_prop_rules_names(act_revert):
    assert list(act_revert._prop_rules()) == [
    "processo_SEI",
            "nome",
       "matricula",
   "cargo_efetivo",
          "classe",
          "padrao",
          "quadro",
"fundamento_legal",
           "orgao",
        "vigencia",
 "matriucla_SIAPE"
    ]

def test_revertions_prop_rules_rules(act_revert):
    assert list(act_revert._prop_rules().values()) == [
    r"(?<!lei)\s((?:[0-9|\s]*?[.|-]\s?)+?[0-9|\s]*/\s?[0-9|\s]*-?\s?[0-9|\s]*)[.|,]",
    r"\s([^,]*?),\smatricula",
    r"matricula\s?n?o?\s([\s\S]*?-[\s\S]*?)[,]",
    r"(?:Cargo|Carreira)\sde([\s\S]*?)\,",
    r"(?:([^,]*?)\sclasse,)?(?(1)|classe\s([\s\S]*?),)",
    r"[p|P]adr[a|ã]o\s([\s\S]*?),",
    r"d?[e|a|o]?(Quadro[\s\S]*?)[,|;|.]",
    r"nos\stermos\sdo\s([\s\S]*?),\sa?\s",
    r"Lotacao:|Quadro\sde\sPessoal\sd[a|e|o]([\s\S]*?)[.|,]",
    "",
    r"siape\sn?o?\s([\s\S]*?)[,| | .]"
    ]

def test_act_revertions_consistence_rule(act_revert):
    assert len(act_revert._props_names())-1 == len(act_revert._prop_rules())

def test_act_revertions_ner():
    act = Revertions(file, 'ner')
    assert act._load_model() == None

#
#
#
#
#

@pytest.fixture()
def act_nomcom():
    return NomeacaoComissionados(file, 'regex')

def test_nomCom_backend(act_nomcom):
    assert act_nomcom._backend == "regex"

def test_nomCom_name(act_nomcom):
    assert act_nomcom._name == "Nomeação"
    assert act_nomcom.name == "Nomeação"
    assert act_nomcom._act_name() == "Nomeação"

def test_nomCom_flags(act_nomcom):
    assert act_nomcom._regex_flags() == 0

def test_nomCom_prop_names(act_nomcom):
    assert act_nomcom._props_names() ==  ['Tipo do Ato', 'Nome', 'Cargo Efetivo', 'Matricula', 'Siape',
                'Simbolo', 'Cargo Comissao', 'Lotacao', 'Orgao']

def test_nomCom_rule(act_nomcom):
    assert act_nomcom._rule_for_inst() == r"(NOMEAR)([\s\S]*?)\.\s"

def test_nomCom_prop_rules_names(act_nomcom):
    assert list(act_nomcom._prop_rules()) == [
              "nome",
     "cargo_efetivo",
         "matricula",
   "matricula_SIAPE",
           "simbolo",
"cargo_comissionado",
"hierarquia_lotacao",
             "orgao"
    ]

def test_nomCom_prop_rules_rules(act_nomcom):
    assert list(act_nomcom._prop_rules().values()) == [
    r"(^[A-ZÀ-Ž\s]+[A-ZÀ-Ž])",
    r"(?:^[A-ZÀ-Ž\s]+[A-ZÀ-Ž]),\s(?![M|m]atr[i|í]cula)([\s\S]*?),\s",
    r"[M|m]atr[í|i]cula\s?n?o?\s([\s\S]*?)[,|\s]",
    r"[S|s][I|i][A|a][P|p][E|e]\s[N|n]?[o|O]?\s([\s\S]*?)[,| | .]",
    r"[S|s][í|i]mbolo\s?n?o?\s([\s\S]*?)[,|\s]",
    r"(?:[S|s][í|i]mbolo\s?n?o?\s(?:[\s\S]*?)[,|\s])\sde([\s\S]*?),",
    r"(?:[S|s][í|i]mbolo\s?n?o?\s(?:[\s\S]*?)[,|\s])\sde(?:[\s\S]*?),\sd[a|e|o]\s([\s\S]*,?),",
    r"(?:[S|s][í|i]mbolo\s?n?o?\s(?:[\s\S]*?)[,|\s])\sde(?:[\s\S]*?),\sd[a|e|o]\s(?:[\s\S]*,?),\sd[a|e|o]\s([\s\S]*?)$"
    ]

def test_act_nomCom_consistence_rule(act_nomcom):
    assert len(act_nomcom._props_names())-1 == len(act_nomcom._prop_rules())


def test_act_nomcom_ner():
    act = NomeacaoComissionados(file, 'ner')
    assert type(act._load_model()) == sklearn_crfsuite.estimator.CRF

#
#
#
#
#


@pytest.fixture()
def act_nom_efet():
    return NomeacaoEfetivos(file, 'regex')

def test_nom_efet_backend(act_nom_efet):
    assert act_nom_efet._backend == "regex"

def test_nom_efet_name(act_nom_efet):
    assert act_nom_efet._name == "Nomeação de Efetivos"
    assert act_nom_efet.name == "Nomeação de Efetivos"
    assert act_nom_efet._act_name() == "Nomeação de Efetivos"

def test_nom_efet_flags(act_nom_efet):
    assert act_nom_efet._regex_flags() == 0

def test_nom_efet_prop_names(act_nom_efet):
    assert act_nom_efet._props_names() ==  ['tipo','edital_normativo','data_do_edital_normativo','DODF_edital_normativo','data_DODF_edital_normativo','edital_resultado_final','data_edital_resultado_final','cargo','especialiade','carreira','orgao','nome_candidato','classificacao','pne','sei','reposicionamento']

def test_nom_efet_rule(act_nom_efet):
    assert act_nom_efet._rule_for_inst() == r"(NOMEAR\s)((?:[ao]s\scandidat[ao]s\sabaixo(?:[a-zA-Z_0-9,\s\/-\:\-\(\);]*).|(?:[ao]\scandidat[oa]\sabaixo(?:[a-zA-Z_0-9,\s\/-\:\-\(\)]*)))(?:\s[a-zA-Z_\s]*(?:deficiencia|especiais):(?:\s[\sA-Zo]+,\s?\d{1,4}o?;?)+)?(?:\s)?(?:[\r\n\t\f\sa-zA-Z_\s]*classificacao:(?:\s[\sA-Zo]+,\s?\d{1,4}o?[,;]?)+)?)"

def test_nom_efet_prop_rules_names(act_nom_efet):
    assert list(act_nom_efet._prop_rules()) == [
    "edital_normativo",
    "data_edital_normativo",
    "numero_dodf_edital_normativo",
    "data_dodf_edital_normativo",
    "edital_resultado_final",
    "data_edital_resultado_final",
    "cargo",
    "especialiade",
    "carreira",
    "orgao",
    "candidato",
    "classificacao",
    "pne",
    "processo_SEI",
    "reposicionamento"
    ]

def test_nom_efet_prop_rules_rules(act_nom_efet):
    assert list(act_nom_efet._prop_rules().values()) == [
    r"Edital\s(?:[Nn]ormativo|de\s[Aa]bertura)\sno\s([\/\s\-a-zA-Z0-9_]+)",
    r"",
    r"publicado\sno\sDODF\sno\s(\d{1,3})",
    r"publicado\sno\sDODF\sno\s\d{1,3},\s?de([\s0-9a-or-vzç]*\d{4})",
    r"Resultado\sFinal\sno\s([\/\s\-a-zA-Z0-9_]+)",
    r"",
    r"DODF\sno\s\d{1,3}(?:,[\s0-9a-or-vzç]*\d{4}),[\sa-z]*([A-Z\s]+)",
    r"[\s,a-z\(\:\)]+([\sA-Z\-]*):",
    r"[cC]arreira\s(?:d[ae]\s)?([a-zA-Z\s]+)",
    r"[cC]arreira\s(?:d[ae]\s)?(?:[a-zA-Z\s]+),\s?d?[ao]?\s?([\sa-zA-Z0-9_]*)",
    r"(?:[\sA-Z\-]*):(?:[\sa-zC]*:)?\s([\sA-Z0-9\,o\;]+)",
    r"",
    r"(?:deficiencia|especiais):\s([\sA-Z0-9\,o\;]+)",
    r"(?<!lei)\s((?:[0-9|\s]*?[.|-]\s?)+?[0-9|\s]*/\s?[0-9|\s]*-?\s?[0-9|\s]*)[.|,]",
    r"lista\sde\sclassificacao:\s([\sA-Z0-9\,o\;]+)"
    ]

def test_act_nom_efet_consistence_rule(act_nom_efet):

    assert len(act_nom_efet._props_names())-1 == len(act_nom_efet._prop_rules())

def test_act_nom_efet_ner():
    act = NomeacaoEfetivos(file, 'ner')
    assert act._load_model() == None



import os
import re
import pytest
import joblib
import sklearn_crfsuite
from unittest.mock import patch

from dodfminer.extract.pure.utils import title_filter

@pytest.fixture(scope='module')
def filter_class():
    return title_filter.BoldUpperCase

def test_dict_text(filter_class):
    # Can be a title
    d = {
        'text': "SECRETARIA DE PESSOAL DO DISTRITO FEDERAL",
    } 
    assert filter_class.dict_text(d) == True

def test_dict_text2(filter_class):
    # Can not be a title since has 4+ consecutive spaces
    d = {
        'text': "SECRETARIA    DE PESSOAL DO DISTRITO FEDERAL",
    } 
    assert filter_class.dict_text(d) == False

def test_dict_text3(filter_class):
    # Can not be a title since has lower case chars
    d = {
        'text': "SECRETARIA DE pessoal DO DISTRITO FEDERAL",
    } 
    assert filter_class.dict_text(d) == False

def test_dict_text4(filter_class):
    # Can not be a title since has 4- chars
    d = {
        'text': "BOM",
    } 
    assert filter_class.dict_text(d) == False

def test_dict_bold(filter_class):
    d = {
        'flags': 14
    }
    assert filter_class.dict_bold(d) == False

def test_dict_bold2(filter_class):
    d = {
        'flags': 16
    }
    assert filter_class.dict_bold(d) == True


def test_dict_bold3(filter_class):
    d = {
        'flags': 18
    }
    assert filter_class.dict_bold(d) == False

def test_dict_bold4(filter_class):
    d = {
        'flags': 20
    }
    assert filter_class.dict_bold(d) == True






"""Command Line Input handler module.

Typical usage example:
    args = CLI().parse()
"""

from dodfminer.__version__ import __version__
from argparse import ArgumentParser

act_choices = ["aposentadoria",
                "reversoes",
                "nomeacao",
                "exoneracao",
                "abono",
                "retificacoes",
                "substituicao",
                "cessoes",
                "sem_efeito_aposentadoria",
                "efetivos_nome",
                "efetivos_exo"]

class CLI(object):
    """CLI Class contains all parameters to handle arguments.

    Set Command Line Input groups and arguments.

    Attributes:
        parser (:obj:`ArgumentParser`): An ArgumentParser object.
        subparsers: Adds subparser to the parser, each one is like a
                    standalone aplication.
        def_start_date (str): Start date to download DODFS. Default start date
                              to download 01/19.
        def_end_date (str): End date to download DODFS. Default end date
                            to download 01/19.
        pure_text (bool): Enable extraction in pure text mode.
                          Defaults to False.
        block (bool): Enable extraction in bloc mode.
                      Defaults to False.
        titles_with_boxes (bool): Enable extraction in titles with boxes mode.
                                  Defaults to False.
        save_path (str): Save path of the download. Defaults to './data'.
        input_folder (str): Path where the extractor should look to files.
                            Defaults to './data'.

    """

    def __init__(self):
        """Init CLI class with default values."""
        desc = """Data extractor of PDF documents from the Official Gazette
                  of the Federal District, Brazil."""
        epilog = f'© Copyright 2020, KnEDLe Team. Version {__version__}'
        self.parser = ArgumentParser(prog="DODFMiner", description=desc,
                                     epilog=epilog)
        self.subparsers = self.parser.add_subparsers(dest='subparser_name')
        self.def_start_date = '01/19'
        self.def_end_date = '01/19'
        self.save_path = './'

    def _new_group(self, name, subparser):
        """Create new argument group.

        Args:
            name: Name of the group.
            subparser: The subparser.

        Returns:
            The argparse group created.

        """
        group = subparser.add_argument_group(name)
        return group

    def _download_parser(self):
        """Create parser for download configs."""
        self.download_parser = self.subparsers.add_parser("downloader")

        help_text = 'Folder to output the download DODFs'
        self.download_parser.add_argument('-sp', '--save_path', dest='save_path',
                                     default=self.save_path, type=str,
                                     help=help_text)

        help_text = 'Input the date in either mm/yyyy or mm-yyyy.'
        self.download_parser.add_argument('-sd', '--start_date', dest='start_date',
                                     default=self.def_start_date, type=str,
                                     help=help_text)

        help_text = 'Input the date in either mm/yyyy or mm-yyyy.'
        self.download_parser.add_argument('-ed', '--end_date', dest='end_date',
                                     default=self.def_end_date, type=str,
                                     help=help_text)

    def _extract_content_parser(self):
        """Create parser for extraction configs."""
        self.extract_content_parser = self.subparsers.add_parser("extract")

        group = self._new_group('Extraction Configs', self.extract_content_parser)

        group.add_argument('-i', '--input-folder', dest='input_folder',
                           default='./', type=str,
                           help='Path to the PDFs folder')

        group.add_argument('-s', '--single-file', dest='single_file', type=str,
                           default=None,
                           help='Path to the single file to extract')

        group.add_argument('-t', '--type-of-extraction', dest='type_of_extr',
                           default=None, type=str, nargs='?',
                           choices=['pure-text', 'blocks', 'with-titles'],
                           help="Type of text extraction")

        group.add_argument('-a', '--act', dest='act', default='all', type=str,
                           choices=act_choices, nargs='*',
                           help='Which acts to extract to CSV')

        group.add_argument('-b', '--backend', dest='backend', default='regex',
                           type=str, choices=['regex', 'ner'],
                           help="The backend to be used in CSV extraction")

        group.add_argument('-x', '--xml', dest='xml', default=False, nargs='*',
                            type=bool, help="Generate TeamTat XML Annotations")

    def parse(self):
        """Create parser and parse the arguments.

        Returns:
            The cli arguments parsed.

        """
        self._download_parser()
        self._extract_content_parser()
        return self.parser.parse_args()



import os
import re
import json
import pytest
import joblib
import inspect
from unittest.mock import patch
from pathlib import Path
from functools import reduce
import operator

import fitz
from dodfminer.extract.pure.utils import title_extractor
from dodfminer.extract.pure.utils.title_extractor import Box, BBox
from dodfminer.extract.pure.utils.title_extractor import TextTypeBboxPageTuple as Tuple


BASE_PATH = Path(
    ""+os.path.dirname(__file__)+"/support"
)

PDF_2020_PATH = BASE_PATH/'03-04-2020_marked.pdf'
PDF_2018_PATH = BASE_PATH/'03-12-2018 INTEGRA.pdf'
PDF_2017_PATH = BASE_PATH/'05-10-2017 INTEGRA.pdf'
PDF_2001_PATH = BASE_PATH/'6-12-2001_secao1.pdf'

BLOCKS_2001 = BASE_PATH/''
BOLD_UPPER_CONTENT = BASE_PATH/''

def jsdump(obj):
    """Simple wrapper for turning object into json.
    Its only purpose is to save some lines."""
    return json.dumps(obj, ensure_ascii=False)

def tuplefy(lis):
    """Turn iterables into tuples. Makes comparison easier."""
    try:
        return tuple([tuplefy(i) for i in lis])
    except:
        return tuple([tuple(i) for i in lis])


def stuplefy(lis):
    """Turn iterable tuple of tuples."""
    return tuple([tuple(i) for i in lis])

@pytest.fixture(scope='module')
def extractor_2001():
    return title_extractor.ExtractorTitleSubtitle(PDF_2001_PATH.as_posix())

@pytest.fixture(scope='module')
def extractor_2017():
    return title_extractor.ExtractorTitleSubtitle(PDF_2017_PATH.as_posix())

@pytest.fixture(scope='module')
def extractor_2018():
    return title_extractor.ExtractorTitleSubtitle(PDF_2018_PATH.as_posix())

@pytest.fixture(scope='module')
def extractor_2020():
    return title_extractor.ExtractorTitleSubtitle(PDF_2020_PATH.as_posix())





@pytest.fixture(scope='module')
def doc_2001():
    return fitz.open(PDF_2001_PATH)

@pytest.fixture(scope='module')
def doc_2017():
    return fitz.open(PDF_2017_PATH)

@pytest.fixture(scope='module')
def doc_2018():
    return fitz.open(PDF_2018_PATH)

@pytest.fixture(scope='module')
def doc_2020():
    return fitz.open(PDF_2020_PATH)

def test_load_blocks_list_1():
    functions_with_path('load_blocks_list', PDF_2001_PATH)

def test_load_blocks_list_2():
    functions_with_path('load_blocks_list', PDF_2017_PATH)

def test_load_blocks_list_3():
    functions_with_path('load_blocks_list', PDF_2018_PATH)

def test_load_blocks_list_4():
    functions_with_path('load_blocks_list', PDF_2020_PATH)


def functions_with_path(name, pdf_path):
    """Wrapper for member functions of title_extractor which
         expects to receive a `pathlib.Path` (indidicating a file path).
    """
    assert (
        jsdump(getattr(title_extractor, name)(pdf_path))
        == 
        jsdump(json.load(open(
            pdf_path.as_posix()[:-4] + '/' f'{name}.json'))
        )
    )

def functions_with_kargs(name, pdf_path, **kargs):
    """Wrapper for member functions of title_extractor which
         expects to receive a `pathlib.Path` (indidicating a file path).
    """
    assert (
        jsdump(getattr(title_extractor, name)(**kargs))
        == 
        jsdump(json.load(open(
            pdf_path.as_posix()[:-4] + '/' f'{name}.json'))
        )
    )


def functions_whole_doc(name, doc):
    assert (
        jsdump(getattr(title_extractor, name)(doc))
        == 
        jsdump(json.load(open(
            doc.name[:-4] + '/' f'{name}.json'))
        )
    )




@pytest.fixture(scope='module')
def elements_and_width():
    return ([
        BBox(Box(x0=100, y0=110, x1=150, y1=100)),
        BBox(Box(x0=200, y0=200, x1=150, y1=100)),
        BBox(Box(x0=300, y0=50, x1=150, y1=100)),
        BBox(Box(x0=400, y0=20, x1=150, y1=100)),

        BBox(Box(x0=500, y0=400, x1=150, y1=100)),
        BBox(Box(x0=600, y0=678, x1=150, y1=100)),
        BBox(Box(x0=700, y0=551, x1=150, y1=100)),
        BBox(Box(x0=800, y0=100, x1=150, y1=100)),
        BBox(Box(x0=900, y0=240, x1=150, y1=100)),
    ], 900)

@pytest.fixture()
def elements_expected():
    return [
        [        
            BBox(Box(x0=100, y0=110, x1=150, y1=100)),
            BBox(Box(x0=200, y0=200, x1=150, y1=100)),
            BBox(Box(x0=300, y0=50, x1=150, y1=100)),
            BBox(Box(x0=400, y0=20, x1=150, y1=100)),
        ],
        [
            BBox(Box(x0=500, y0=400, x1=150, y1=100)),
            BBox(Box(x0=600, y0=678, x1=150, y1=100)),
            BBox(Box(x0=700, y0=551, x1=150, y1=100)),
            BBox(Box(x0=800, y0=100, x1=150, y1=100)),
            BBox(Box(x0=900, y0=240, x1=150, y1=100)),        ],
    ]


def test_group_by_column(elements_and_width, elements_expected):
    grouped = title_extractor.group_by_column(*elements_and_width)
    assert stuplefy(grouped) == stuplefy(elements_expected)


@pytest.fixture()
def elements_page():
    return [
        Tuple('texto', None, None, 1),
        Tuple('outro texto', None, None, 2),
        Tuple('ficar junto com "texto"', None, None, 1),
        Tuple('tururu', None, None, 14),
        Tuple('just another text', None, None, 55),
        Tuple('KnEDLe rocks!', None, None, 3),
        Tuple('Grande Teo', None, None, 55),
        Tuple('>>> Vic', None, None, 3),
    ]

@pytest.fixture()
def elements_page_expected():
    return {
        1: [
            Tuple('texto', None, None, 1),
            Tuple('ficar junto com "texto"', None, None, 1),
        ],
        2: [
            Tuple('outro texto', None, None, 2),
        ],
        3: [
            Tuple('KnEDLe rocks!', None, None, 3),
            Tuple('>>> Vic', None, None, 3),
        ],
        14: [
            Tuple('tururu', None, None, 14),
        ],
        55: [
            Tuple('just another text', None, None, 55),
            Tuple('Grande Teo', None, None, 55),
        ],
    }

def test_group_by_page(elements_page, elements_page_expected):
    grouped = title_extractor.group_by_page(elements_page)
    assert (sorted(grouped.keys()) == sorted(elements_page_expected.keys())
            and json.dumps(elements_page_expected) == json.dumps(grouped)
    )


@pytest.fixture()
def elements_expected_sort():
    return reduce(operator.add, (
        [        
            BBox(Box(x0=400, y0=20, x1=150, y1=100)),
            BBox(Box(x0=300, y0=50, x1=150, y1=100)),
            BBox(Box(x0=100, y0=110, x1=150, y1=100)),
            BBox(Box(x0=200, y0=200, x1=150, y1=100)),
        ],
        [
            BBox(Box(x0=800, y0=100, x1=150, y1=100)),
            BBox(Box(x0=900, y0=240, x1=150, y1=100)),
            BBox(Box(x0=500, y0=400, x1=150, y1=100)),
            BBox(Box(x0=700, y0=551, x1=150, y1=100)),
            BBox(Box(x0=600, y0=678, x1=150, y1=100)),
        ]
    ))


def test_sort_by_column(elements_and_width, elements_expected_sort):
    sorted_ = title_extractor.sort_by_column( *elements_and_width )
    assert json.dumps(elements_expected_sort) == json.dumps(sorted_)


def test_invert_TextTypeBboxPageTuple_1():
    tup = Tuple('texto', title_extractor._TYPE_SUBTITLE, 123, 34)
    inv_tup = Tuple('texto', title_extractor._TYPE_TITLE, 123, 34)
    assert inv_tup == title_extractor._invert_TextTypeBboxPageTuple(tup)

def test_invert_TextTypeBboxPageTuple_2():
    tup = Tuple('nadaver', title_extractor._TYPE_TITLE, 123, 34)
    inv_tup = Tuple('nadaver', title_extractor._TYPE_SUBTITLE, 123, 34)
    assert inv_tup == title_extractor._invert_TextTypeBboxPageTuple(tup)

def test_invert_TextTypeBboxPageTuple_3():
    tup = Tuple('nadaver', title_extractor._TYPE_TITLE, 123, 34)
    inv_tup = Tuple('nadaperder', title_extractor._TYPE_SUBTITLE, 123, 34)
    assert inv_tup != title_extractor._invert_TextTypeBboxPageTuple(tup)


def test_extract_bold_upper_page():
    """ '_extract_bold_upper_page' will not be tested directly"""
    pass

@pytest.mark.xfail(reason="Bug #45 not yet fixed.", run=False)
def test_extract_bold_upper_pdf_1(doc_2001):
    functions_whole_doc('_extract_bold_upper_pdf', doc_2001)

@pytest.mark.xfail(reason="Bug #45 not yet fixed.", run=False)
def test_extract_bold_upper_pdf_2(doc_2017):
    functions_whole_doc('_extract_bold_upper_pdf', doc_2017)

@pytest.mark.xfail(reason="Bug #45 not yet fixed.", run=False)
def test_extract_bold_upper_pdf_3(doc_2018):
    functions_whole_doc('_extract_bold_upper_pdf', doc_2018)

@pytest.mark.xfail(reason="Bug #45 not yet fixed.", run=False)
def test_extract_bold_upper_pdf_4(doc_2020):
    functions_whole_doc('_extract_bold_upper_pdf', doc_2020)


def test_sort_2column():
    """ 'sort_2column' will not be tested directly.
    It is just a wrapper using `group_by_page` and `sort_by_column`
    """
    pass


def test_get_titles_subtitles():
    """Hard to test. It's indirectly tested by `test_get_titles_subtitles_smart_X`"""
    pass

@pytest.mark.xfail(reason="Bug #45 not yet fixed (will change function signature).", run=False)
def test_get_titles_subtitles_smart_1(doc_2001):
    functions_with_kargs(
        '_get_titles_subtitles_smart', PDF_2001_PATH, doc=doc_2001,
        width_lis = [p.MediaBox[2] for p in doc_2001],
    )
@pytest.mark.xfail(reason="Bug #45 not yet fixed (will change function signature).", run=False)
def test_get_titles_subtitles_smart_2(doc_2017):
    functions_with_kargs(
        '_get_titles_subtitles_smart', PDF_2017_PATH, doc=doc_2017,
        width_lis = [p.MediaBox[2] for p in doc_2017],
    )
@pytest.mark.xfail(reason="Bug #45 not yet fixed (will change function signature).", run=False)
def test_get_titles_subtitles_smart_3(doc_2018):
    functions_with_kargs(
        '_get_titles_subtitles_smart', PDF_2018_PATH, doc=doc_2018,
        width_lis = [p.MediaBox[2] for p in doc_2018],
    )
@pytest.mark.xfail(reason="Bug #45 not yet fixed (will change function signature).", run=False)
def test_get_titles_subtitles_smart_4(doc_2020):
    functions_with_kargs(
        '_get_titles_subtitles_smart', PDF_2020_PATH, doc=doc_2020,
        width_lis = [p.MediaBox[2] for p in doc_2020],
    )


def test_extract_titles_subtitles_1():
    functions_with_path('extract_titles_subtitles', PDF_2001_PATH)

def test_extract_titles_subtitles_2():
    functions_with_path('extract_titles_subtitles', PDF_2017_PATH)

def test_extract_titles_subtitles_3():
    functions_with_path('extract_titles_subtitles', PDF_2018_PATH)

@pytest.mark.xfail(reason="Bug #45 not yet fixed.", run=False)
def test_extract_titles_subtitles_4():
    functions_with_path('extract_titles_subtitles', PDF_2020_PATH)



def test_titles_1(extractor_2001):
    wrapper_extractor_props(extractor_2001, 'titles')

def test_titles_2(extractor_2017):
    wrapper_extractor_props(extractor_2017, 'titles')

def test_titles_3(extractor_2018):
    wrapper_extractor_props(extractor_2018, 'titles')

@pytest.mark.xfail(reason="Bug #45 not yet fixed.", run=False)
def test_titles_4(extractor_2020):
    wrapper_extractor_props(extractor_2020, 'titles')


def test_subtitles_1(extractor_2001):
    wrapper_extractor_props(extractor_2001, 'subtitles')

def test_subtitles_2(extractor_2017):
    wrapper_extractor_props(extractor_2017, 'subtitles')

def test_subtitles_3(extractor_2018):
    wrapper_extractor_props(extractor_2018, 'subtitles')

@pytest.mark.xfail(reason="Bug #45 not yet fixed.", run=False)
def test_subtitles_4(extractor_2020):
    wrapper_extractor_props(extractor_2020, 'subtitles')


def test_json_1(extractor_2001):
    wrapper_extractor_props(extractor_2001, 'json')

def test_json_2(extractor_2017):
    wrapper_extractor_props(extractor_2017, 'json')

def test_json_3(extractor_2018):
    wrapper_extractor_props(extractor_2018, 'json')

@pytest.mark.xfail(reason="Bug #45 not yet fixed.", run=False)
def test_json_4(extractor_2020):
    wrapper_extractor_props(extractor_2020, 'json')



def wrapper_extractor_props(extractor, name):
    ground_truth = json.dumps(json.load(
        open(Path(extractor._path[:-4])/f'{name}.json'))
    )
    hier = json.dumps(getattr(extractor, name))
    assert ground_truth == hier


def test_titles_subtitles_hierarchy_1(extractor_2001):
    wrapper_extractor_props(extractor_2001, 'titles_subtitles_hierarchy')

def test_titles_subtitles_hierarchy_2(extractor_2017):
    wrapper_extractor_props(extractor_2017, 'titles_subtitles_hierarchy')

def test_titles_subtitles_hierarchy_3(extractor_2018):
    wrapper_extractor_props(extractor_2018, 'titles_subtitles_hierarchy')


@pytest.mark.xfail(reason="Bug #45 not yet fixed.", run=False)
def test_titles_subtitles_hierarchy_4(extractor_2020):
    wrapper_extractor_props(extractor_2020, 'titles_subtitles_hierarchy')



def test_reset(extractor_2001):
    extractor_2001.reset()
    assert (not extractor_2001._json
            and not extractor_2001._hierarchy
            and not extractor_2001._cache
    )





import os
import pandas as pd

from dodfminer.extract.polished.helper import xml_multiple, get_files_path, print_dataframe, build_act_txt

def test_helper_xml_multiple():
    dir = ""+os.path.dirname(__file__)+"/support/support_supporter"
    try:
        xml_multiple(dir, "regex")
        assert "1_1.2019.xml" in os.listdir(dir)
        os.remove(os.path.join(dir, "1_1.2019.xml"))
    except:
        assert False

# def test_helper_extract_multiple_acts():
#     extract_multiple_acts(path, types, backend)

# def test_helper_extract_multiple():
#     extract_multiple(files, type, backend, txt_out=False, txt_path="./results")

# def test_helper_extract_single():
#     extract_single(file, type, backend)

def test_helper_build_act_txt():
    dir = ""+os.path.dirname(__file__)+"/support/"
    build_act_txt(["aposentadoria"], "crioutxt", save_path=dir)
    assert "crioutxt.txt" in os.listdir(dir)
    os.remove(os.path.join(dir, "crioutxt.txt"))

# def test_helper_print_dataframe():
#     df = print_dataframe(pd.DataFrame())
#     assert isinstance(df, pd.io.formats.style.Styler)

def test_helper_get_files_path():
    dir = ""+os.path.dirname(__file__)+"/support/support_supporter/"
    files = get_files_path(dir, "txt")
    assert files == [""+os.path.dirname(__file__)+'/support/support_supporter/valid.txt']
    files = get_files_path(dir, "pdf")
    assert files == [""+os.path.dirname(__file__)+'/support/support_supporter/DODF 001 01-01-2019 EDICAO ESPECIAL.pdf']




import os
import re
import json
import pytest
import joblib
import sklearn_crfsuite
from unittest.mock import patch
from pathlib import Path

import fitz
from dodfminer.extract.pure.utils import box_extractor

BASE_PATH = Path(
    ""+os.path.dirname(__file__)+"/support"
)
PDF_PATH = BASE_PATH/'03-12-2018.pdf'
PAGE_ONE_LINES_PATH = BASE_PATH/'03-12-2018_pagina1_lines.json'
PAGE_49_LINES_PATH = BASE_PATH/'03-12-2018_pagina49_lines.json'
GET_DOC_TEXT_BOXES_PATH = BASE_PATH/'03-12-2018_get_doc_text_boxes.json'
GET_DOC_TEXT_LINES_PATH = BASE_PATH/'03-12-2018_get_doc_text_lines.json'
GET_DOC_IMG_PATH = BASE_PATH/'03-12-2018_get_doc_img.json'
GET_DOC_IMG_BOXES_PATH = BASE_PATH/'03-12-2018_get_doc_img_boxes.json'

def tuplefy(lis):
    """Turn iterables into tuples. Makes comparison easier."""
    try:
        return tuple([tuplefy(i) for i in lis])
    except:
        return tuple([tuple(i) for i in lis])

@pytest.fixture(scope='module')
def pdf_fitz():
    return fitz.open(PDF_PATH.as_posix())

def wrapper_extract_page_lines_content(pdf_fitz, page_num, path):
    # Can be a title
    as_tup = tuplefy(
        box_extractor._extract_page_lines_content(pdf_fitz[page_num])
    )
    ground_truth = json.load(open(path))
    ground_truth = tuplefy(ground_truth)
    assert as_tup == ground_truth


def test_extract_page_lines_content_1(pdf_fitz):
    # Can be a title
    wrapper_extract_page_lines_content(
        pdf_fitz, 0, PAGE_ONE_LINES_PATH
    )


def test_extract_page_lines_content_49(pdf_fitz):
    # Can be a title
    wrapper_extract_page_lines_content(
        pdf_fitz, 48, PAGE_49_LINES_PATH
    )


def test_get_doc_text_boxes(pdf_fitz):
    ground_truth = json.load(
        open(GET_DOC_TEXT_BOXES_PATH.as_posix())
    )
    assert (
        tuplefy(ground_truth) 
        == tuplefy(box_extractor.get_doc_text_boxes(pdf_fitz))
    )


def test_doc_text_lines(pdf_fitz):
    ground_truth = json.load(
        open(GET_DOC_TEXT_LINES_PATH.as_posix())
    )
    assert (
        tuplefy(ground_truth) 
        == tuplefy(box_extractor.get_doc_text_lines(pdf_fitz))
    )


def test_get_doc_img(pdf_fitz):
    ground_truth = json.load(
        open(GET_DOC_IMG_PATH.as_posix())
    )
    assert (
        tuplefy(ground_truth) 
        == tuplefy(box_extractor._get_doc_img(pdf_fitz))
    )


def test_get_doc_img(pdf_fitz):
    ground_truth = json.load(
        open(GET_DOC_IMG_PATH.as_posix())
    )
    assert (
        tuplefy(ground_truth) 
        == tuplefy(box_extractor._get_doc_img(pdf_fitz))
    )


def test_get_doc_img_boxes(pdf_fitz):
    ground_truth = json.load(
        open(GET_DOC_IMG_BOXES_PATH.as_posix())
    )
    assert (
        tuplefy(ground_truth) 
        == tuplefy(box_extractor.get_doc_img_boxes(pdf_fitz))
    )





import pytest
import numpy as np
from unittest.mock import patch
from dodfminer.extract.polished.backend.regex import ActRegex

@pytest.fixture
@patch.object(ActRegex, '_rule_for_inst',
              return_value=r"(MENSAGEM:)([^.]*)")
@patch.object(ActRegex, '_prop_rules',return_value={
    "numeros": r"([0-9]+)",
    "capitalizado": r"([A-Z][a-z]+)",
    "nao_encontra": "khalil"
})
def act_regex(test_rule_for_inst, test_prop_rules):
    act =  ActRegex()
    act._acts_str = []
    return act

def test_act_regex_flags(act_regex):
    assert act_regex._flags == 0

def test_act_regex_rules(act_regex):
    assert act_regex._rules == {
    "numeros": r"([0-9]+)",
    "capitalizado": r"([A-Z][a-z]+)",
    "nao_encontra": "khalil"
    }

def test_act_regex_inst_rule(act_regex):
    assert act_regex._inst_rule == r"(MENSAGEM:)([^.]*)"

def test_act_regex_regex_props(act_regex):
    act_regex._name = "Teste"
    act_raw = "MENSAGEM: O Renato testa esse codigo 1232 vezes por dia."
    act = act_regex._regex_props(act_raw)
    assert act == {"tipo_ato": "Teste", "numeros": "1232", "capitalizado": "Renato", "nao_encontra": np.nan}


def test_act_regex_regex_instances(act_regex):
    act_regex._text = "fghngfnfgnfgnfgnrr MENSAGEM: O Renato testa esse codigo 1232 vezes por dia. qeqcnoecqucqpwxqrthrweqqeprto MENSAGEM: O Joao testa esse codigo 2 vezes por dia. wsfhsn wenrwermrkne MENSAGEM: O Lucas testa esse codigo 0 vezes por dia. efwefwefwfwefwetrynyujuju"
    act_regex.acts_str = []
    act = act_regex._regex_instances()
    assert act == [' O Renato testa esse codigo 1232 vezes por dia', ' O Joao testa esse codigo 2 vezes por dia', ' O Lucas testa esse codigo 0 vezes por dia']

def test_act_regex_find_prop_value(act_regex):
    res = act_regex._find_prop_value("(de)", "String de teste")
    assert res == ('de',)

def test_act_regex_find_multiple_prop_value(act_regex):
    res = act_regex._find_prop_value("(de) (teste)", "String de teste")
    assert res == ('de', 'teste',)

def test_act_regex_dont_find_prop_value(act_regex):
    res = act_regex._find_prop_value("dos", "String de teste")
    assert res is np.nan

@patch.object(ActRegex, '_prop_rules')
def test_no_rule_inst_act_regex(test_prop_rules):
    with pytest.raises(Exception):
        ActRegex()

@patch.object(ActRegex, '_rule_for_inst')
def test_no_prop_rule_act_regex(test_rule_for_inst):
    with pytest.raises(Exception):
        ActRegex()

def test_not_implemented_functions_act_regex():
    with pytest.raises(Exception):
        ActRegex()



version_info = (1, 3, 5)
# format:
# ('dodf_major', 'dodf_minor', 'dodf_patch')


def get_version():
    "Returns the version as a human-format string."
    return '%d.%d.%d' % version_info


__version__ = get_version()



import pytest
from dodfminer.downloader.helper import req1, req2, req3, get_downloads, check_date   


def test_req1():
    assert req1(2017) == 'https://dodf.df.gov.br/listar?dir=2017' 

def test_req2():
    assert  type(req2('https://dodf.df.gov.br/listar?dir=2017', '01_Janeiro')) == tuple

def test_req2_2():
    with pytest.raises(ValueError):
        assert (req2('https://dodf.df.gov.br/listar?dir=2017', 'Janeiro'))

def test_req3():
    assert  type(req3('https://dodf.df.gov.br/listar?dir=2017', 'DODF 023 31-01-2006')) == tuple

def test_get_downloads():
    assert type(get_downloads('2017', '01_Janeiro')) == dict

def test_get_downloads_2():
    with pytest.raises(ValueError):
        assert (get_downloads('2017', 'Janeiro'))

def test_check_date():
    assert type(check_date('2017', '01_Janeiro')) == bool

def test_check_date_2():
    with pytest.raises(ValueError):
        assert (check_date('2017', 'Janeiro'))


import os
import pytest
import numpy as np
import pandas as pd
from unittest.mock import patch
from dodfminer.extract.polished.acts.base import Atos

rule_dict = {
    "numeros": r"([0-9]+)",
    "capitalizado": r"([A-Z][a-z]+)",
    "nao_encontra": "khalil"
}
rule_prop = r"(MENSAGEM:)([^.]*)"
props = ["Tipo", "Numeros", "Capitalizado", "Não Encontra"]
valid_file = ""+os.path.dirname(__file__)+"/support/valid.txt"

text = "fghngfnfgnfgnfgnrr MENSAGEM: O Renato testa esse codigo 1232 vezes por dia. qeqcnoecqucqpwxqrthrweqqeprto MENSAGEM: O Joao testa esse codigo 2 vezes por dia. wsfhsn wenrwermrkne MENSAGEM: O Lucas testa esse codigo 0 vezes por dia. efwefwefwfwefwetrynyujuju"
act =  [' O Renato testa esse codigo 1232 vezes por dia', ' O Joao testa esse codigo 2 vezes por dia', ' O Lucas testa esse codigo 0 vezes por dia']

res_act = [{'tipo_ato': 'Aposentadoria', 'numeros': '1232', 'capitalizado': 'Renato', 'nao_encontra': np.nan}, {'tipo_ato': 'Aposentadoria', 'numeros': '2', 'capitalizado': 'Joao', 'nao_encontra': np.nan}, {'tipo_ato': 'Aposentadoria', 'numeros': '0', 'capitalizado': 'Lucas', 'nao_encontra': np.nan}]


# Wrong Init Type
@patch.object(Atos, '_act_name', return_value="Aposentadoria")
@patch.object(Atos, '_props_names', return_value=props)
@patch.object(Atos, '_rule_for_inst', return_value=rule_prop)
@patch.object(Atos, '_prop_rules', return_value=rule_dict)
def test_act_base_invalid_file(test_act_name, test_return_value,
                               test_rules_inst, test_rules_prop):
    file = ""+os.path.dirname(__file__)+"/support/invalid.pdf"
    act = Atos(file,'regex')
    assert act._file_name == None
    assert act._text == file

@patch.object(Atos, '_act_name', return_value="Aposentadoria")
@patch.object(Atos, '_props_names', return_value=props)
@patch.object(Atos, '_rule_for_inst', return_value=rule_prop)
@patch.object(Atos, '_prop_rules', return_value=rule_dict)
def test_act_non_existiong_backend(test_act_name, test_return_value,
                                   test_rules_inst, test_rules_prop):
    ato = Atos(valid_file, 'lol')
    assert ato._backend == 'regex'

@patch.object(Atos, '_act_name', return_value="Aposentadoria")
@patch.object(Atos, '_props_names', return_value=props)
@patch.object(Atos, '_rule_for_inst', return_value=rule_prop)
@patch.object(Atos, '_prop_rules', return_value=rule_dict)
def test_act_backend_fallback(test_act_name, test_return_value,
                                   test_rules_inst, test_rules_prop):
    ato = Atos(valid_file, 'ner')
    assert ato._backend == 'regex'

# Wrong Init
@patch.object(Atos, '_act_name', return_value="Aposentadoria")
@patch.object(Atos, '_props_names', return_value=props)
@patch.object(Atos, '_rule_for_inst', return_value=rule_prop)
def test_no_overwrite_1(test_act_name, test_return_value,
                        test_rules_inst):
    with pytest.raises(Exception):
        Atos(valid_file,'regex')

@patch.object(Atos, '_act_name', return_value="Aposentadoria")
@patch.object(Atos, '_props_names', return_value=props)
@patch.object(Atos, '_prop_rules', return_value=rule_dict)
def test_no_overwrite_2(test_act_name, test_return_value,
                        test_rules_prop):
    with pytest.raises(Exception):
        Atos(valid_file,'regex')

@patch.object(Atos, '_act_name', return_value="Aposentadoria")
@patch.object(Atos, '_rule_for_inst', return_value=rule_prop)
@patch.object(Atos, '_prop_rules', return_value=rule_dict)
def test_no_overwrite_3(test_act_name, test_rules_inst,
                        test_rules_prop):
    with pytest.raises(Exception):
        Atos(valid_file,'regex')

@patch.object(Atos, '_props_names', return_value=props)
@patch.object(Atos, '_rule_for_inst', return_value=rule_prop)
@patch.object(Atos, '_prop_rules', return_value=rule_dict)
def test_no_overwrite_4(test_return_value, test_rules_inst,
                        test_rules_prop):
    with pytest.raises(Exception):
        Atos(valid_file,'regex')

def test_no_overwrite_5():
    with pytest.raises(Exception):
        Atos(valid_file,'regex')


@pytest.fixture
@patch.object(Atos, '_act_name', return_value="Aposentadoria")
@patch.object(Atos, '_props_names', return_value=["Numeros", "Capitalizado", "Não Encontra"])
@patch.object(Atos, '_rule_for_inst',
              return_value=r"(MENSAGEM:)([^.]*)")
@patch.object(Atos, '_prop_rules',return_value={
    "numeros": r"([0-9]+)",
    "capitalizado": r"([A-Z][a-z]+)",
    "nao_encontra": "khalil"
})
def act_base_regex(test_act_name, test_return_value,
                   test_rules_inst, test_rules_prop):
    atos = Atos(valid_file,'regex')
    atos._name = "Aposentadoria"
    return atos

def test_act_base_name(act_base_regex):
    assert act_base_regex.name == "Aposentadoria"

def test_act_base_name_attr(act_base_regex):
    assert act_base_regex._name == "Aposentadoria"

def test_act_file_name(act_base_regex):
    assert act_base_regex._file_name == valid_file

def test_act_base_raw_acts_regex(act_base_regex):
    act_base_regex._backend = 'regex'
    act_base_regex._text = text
    assert act_base_regex._extract_instances() == act

def test_act_base_raw_acts_ner(act_base_regex):
    act_base_regex._backend = 'ner'
    act_base_regex._text = text
    assert act_base_regex._extract_instances() == act

def test_act_base_raw_acts_error(act_base_regex):
    act_base_regex._backend = 'lol'
    act_base_regex._text = text
    with pytest.raises(Exception):
        act_base_regex._extract_instances()

def test_act_base_props_regex(act_base_regex):
    act_base_regex._backend = 'regex'
    act_base_regex._raw_acts = act
    print(act_base_regex._acts)
    assert act_base_regex._extract_props() == res_act

def test_act_base_build_dataframes(act_base_regex):
    act_base_regex._backend = 'regex'
    act_base_regex._acts = res_act
    act_base_regex._columns = props
    res_df = pd.DataFrame(res_act)
    res_df.columns = props

    assert act_base_regex._build_dataframe().equals(res_df)

def test_act_base_build_dataframes_empty(act_base_regex):
    act_base_regex._backend = 'regex'
    res_df = pd.DataFrame()
    assert act_base_regex._build_dataframe().equals(res_df)

def test_act_base_props_error(act_base_regex):
    act_base_regex._backend = 'lol'
    act_base_regex._raw_acts = act
    with pytest.raises(Exception):
        act_base_regex._extract_props()



import os
import re
import pytest
import joblib
import sklearn_crfsuite
import json
import sys
from glob import glob
import shutil
from unittest.mock import patch
from dodfminer.run import Miner
from dodfminer.run import run

empty_args_msg = """usage: DODFMiner [-h] {downloader,extract} ...

Data extractor of PDF documents from the Official Gazette of the Federal
District, Brazil.

positional arguments:
  {downloader,extract}

optional arguments:
  -h, --help            show this help message and exit

© Copyright 2020, KnEDLe Team. Version 1.1.3.4
"""

only_extract_msg = "usage: DODFMiner extract"

act_choices = ["aposentadoria",
                "nomeacao",
                "exoneracao",
                "abono",
                "retificacoes",
                "reversoes",
                "substituicao",
                "efetivos_nome",
                "efetivos_exo"]

def test_run_no_args(capsys):
  with pytest.raises(SystemExit):
    with patch.object(sys, 'argv', ['cmd', '']):
      run()

def test_run_only_extract_arg(capsys):
  targets = ['cmd', 'extract']
  with patch.object(sys, 'argv', targets):
    run()
    captured = capsys.readouterr()
    assert only_extract_msg in captured.out

def test_run_extract_single_file_pure_text():
  file = ""+os.path.dirname(__file__)+"/support/dodfminer_sf.pdf"
  targets = ["cmd", "extract", "-s", file, "-t", "pure-text"]
  content_txt = "NOMEAR JOSE ANTONIO BARBOSA, Tecnico de Gestao Educacional,"
  with patch.object(sys, 'argv', targets):
    run()
    assert os.path.exists(file.replace('pdf', 'txt'))
    assert content_txt in open(file.replace('pdf', 'txt')).read()
    os.remove(file.replace('pdf', 'txt'))

def test_run_extract_single_file_with_titles():
  file = ""+os.path.dirname(__file__)+"/support/dodfminer_sf.pdf"
  targets = ["cmd", "extract", "-s", file, "-t", "with-titles"]
  with patch.object(sys, 'argv', targets):
    run()
    assert os.path.exists(file.replace('pdf', 'json'))
    res_dict = json.loads(open(file.replace('pdf', 'json')).read())
    assert "PODER EXECUTIVO" in res_dict.keys()
    os.remove(file.replace('pdf', 'json'))

def test_run_extract_single_file_blocks():
  file = ""+os.path.dirname(__file__)+"/support/dodfminer_sf.pdf"
  targets = ["cmd", "extract", "-s", file, "-t", "blocks"]
  with patch.object(sys, 'argv', targets):
    run()
    assert os.path.exists(file.replace('pdf', 'json'))
    res_dict = json.loads(open(file.replace('pdf', 'json')).read())
    assert len(res_dict) > 0
    os.remove(file.replace('pdf', 'json'))

def test_run_extract_single_all_act():
  folder = ""+os.path.dirname(__file__)+"/support/"
  file = folder+"dodfminer_sf.pdf"
  targets = ["cmd", "extract", "-s", file, "-a"]
  with patch.object(sys, 'argv', targets):
    run()
    for act in act_choices:
      assert os.path.isfile(folder+act+".csv")
      os.remove(folder+act+".csv")

def test_run_extract_input_folder_pure_text():
  folder = ""+os.path.dirname(__file__)+"/support/dodf_pdfs"
  targets = ["cmd", "extract", "-i", folder, "-t", "pure-text"]
  with patch.object(sys, 'argv', targets):
    run()
    res_folder = folder + '/results/txt/2020/01_Janeiro/'
    assert os.path.isdir(res_folder)
    assert len(glob(res_folder+'*.txt')) > 1
    shutil.rmtree(folder + '/results/')

def test_run_extract_input_folder_with_titles():
  folder = ""+os.path.dirname(__file__)+"/support/dodf_pdfs"
  targets = ["cmd", "extract", "-i", folder, "-t", "with-titles"]
  with patch.object(sys, 'argv', targets):
    run()
    res_folder = folder + '/results/json/2020/01_Janeiro/'
    assert os.path.isdir(res_folder)
    assert len(glob(res_folder+'*.json')) > 1
    shutil.rmtree(folder + '/results/')

def test_run_extract_input_folder_one_act():
  folder = ""+os.path.dirname(__file__)+"/support/dodf_pdfs"
  targets = ["cmd", "extract", "-i", folder, "-a", "aposentadoria"]
  with patch.object(sys, 'argv', targets):
    run()
    assert os.path.isfile(folder+"/aposentadoria.csv")
    os.remove(folder+"/aposentadoria.csv")

def test_run_extract_input_folder_two_act():
  folder = ""+os.path.dirname(__file__)+"/support/dodf_pdfs"
  targets = ["cmd", "extract", "-i", folder, "-a", "aposentadoria", "abono"]
  with patch.object(sys, 'argv', targets):
    run()
    assert os.path.isfile(folder+"/aposentadoria.csv")
    assert os.path.isfile(folder+"/abono.csv")
    os.remove(folder+"/aposentadoria.csv")
    os.remove(folder+"/abono.csv")

def test_run_extract_input_folder_all_act():
  folder = ""+os.path.dirname(__file__)+"/support/dodf_pdfs/"
  targets = ["cmd", "extract", "-i", folder, "-a"]
  with patch.object(sys, 'argv', targets):
    run()
    for act in act_choices:
      assert os.path.isfile(folder+act+".csv")
      os.remove(folder+act+".csv")
  shutil.rmtree(folder + '/results')

def test_run_extract_input_folder_one_act_back_end_ner():
  folder = ""+os.path.dirname(__file__)+"/support/dodf_pdfs"
  targets = ["cmd", "extract", "-i", folder, "-a", "aposentadoria", "-b", "ner"]
  with patch.object(sys, 'argv', targets):
    run()
    assert os.path.isfile(folder+"/aposentadoria.csv")
    os.remove(folder+"/aposentadoria.csv")

def test_run_extract_input_folder_xml():
  folder = ""+os.path.dirname(__file__)+"/support/xml_extract"
  targets = ["cmd", "extract", "-i", folder, "-x"]
  with patch.object(sys, 'argv', targets):
    run()
    assert os.path.isfile(os.path.join(folder, '1_1.10.1.2020.xml'))
    os.remove(folder+'/1_1.10.1.2020.xml')

# def test_run_extract_input_single_xml():
#   file = ""+os.path.dirname(__file__)+"/support/xml_extract/DODF 001 02-01-2020 INTEGRA.pdf"
#   targets = ["cmd", "extract", "-s", file, "-x"]
#   with patch.object(sys, 'argv', targets):
#     run()
#     res_file = ""+os.path.dirname(__file__)+"/support/xml_extract/1_1.2.1.2020.xml"
#     assert os.path.isfile(res_file)
#     os.remove(res_file)



import setuptools
from dodfminer.__version__ import __version__

def requirements_list():
    list_of_req = []
    with open('requirements.txt') as req:
        for line in req:
            list_of_req.append(line)

    return list_of_req


with open("README.md", "r") as fh:
    long_description = fh.read()

print(requirements_list())

setuptools.setup(
    name="dodfminer",
    version=__version__,
    author="KnEDLe",
    author_email="author@example.com",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/UnB-KnEDLe/DODFMiner",
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': ['dodfminer=dodfminer.run:run'],
    },
    install_requires=requirements_list(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    package_data={'dodfminer': ['extract/polished/acts/models/*.pkl']},
    include_package_data = True,
)



import os

from dodfminer.extract.polished.core import ActsExtractor, _acts_ids

from dodfminer.extract.polished.acts.aposentadoria import Retirements, RetAposentadoria
from dodfminer.extract.polished.acts.nomeacao import NomeacaoComissionados, NomeacaoEfetivos
from dodfminer.extract.polished.acts.exoneracao import Exoneracao, ExoneracaoEfetivos
from dodfminer.extract.polished.acts.reversoes import Revertions
from dodfminer.extract.polished.acts.abono import AbonoPermanencia
from dodfminer.extract.polished.acts.substituicao import Substituicao
from dodfminer.extract.polished.acts.cessoes import Cessoes
from dodfminer.extract.polished.acts.sem_efeito_aposentadoria import SemEfeitoAposentadoria

from dodfminer.extract.polished.create_xml import XMLFy

def test_polished_core_acts():
    assert _acts_ids == {"aposentadoria":Retirements,
                            "reversoes": Revertions,
                            "nomeacao": NomeacaoComissionados,
                            "exoneracao": Exoneracao,
                            "abono": AbonoPermanencia,
                            "retificacoes": RetAposentadoria,
                            "substituicao": Substituicao,
                            "efetivos_nome": NomeacaoEfetivos,
                            "efetivos_exo": ExoneracaoEfetivos,
                            "sem_efeito_aposentadoria": SemEfeitoAposentadoria,
                            "cessoes": Cessoes}

def test_polished_core_act_obj():
    assert type(ActsExtractor.get_act_obj("aposentadoria", "", "regex")) == type(Retirements("", "regex"))
    assert type(ActsExtractor.get_act_obj("reversoes", "", "regex")) == type(Revertions("", "regex"))
    assert type(ActsExtractor.get_act_obj("nomeacao", "", "regex")) == type(NomeacaoComissionados("", "regex"))
    assert type(ActsExtractor.get_act_obj("abono", "", "regex")) == type(AbonoPermanencia("", "regex"))
    assert type(ActsExtractor.get_act_obj("retificacoes", "", "regex")) == type(RetAposentadoria("", "regex"))
    assert type(ActsExtractor.get_act_obj("substituicao", "", "regex")) == type(Substituicao("", "regex"))
    assert type(ActsExtractor.get_act_obj("efetivos_nome", "", "regex")) == type(NomeacaoEfetivos("", "regex"))
    assert type(ActsExtractor.get_act_obj("efetivos_exo", "", "regex")) == type(ExoneracaoEfetivos("", "regex"))
    assert type(ActsExtractor.get_act_obj("exoneracao", "", "regex")) == type(Exoneracao("", "regex"))
    assert type(ActsExtractor.get_act_obj("sem_efeito_aposentadoria", "", "regex")) == type(SemEfeitoAposentadoria("", "regex"))
    assert type(ActsExtractor.get_act_obj("cessoes", "", "regex")) == type(Cessoes("", "regex"))
    assert len(_acts_ids) == 11

def test_polished_core_get_all_obj():
    objs = ActsExtractor.get_all_obj("", "regex")
    needed = {
        "abono": AbonoPermanencia("","regex"),
        "aposentadoria": Retirements("","regex"),
        "efetivos_exo": ExoneracaoEfetivos("","regex"),
        "efetivos_nome": NomeacaoEfetivos("","regex"),
        "exoneracao": Exoneracao("","regex"),
        "nomeacao": NomeacaoComissionados("","regex"),
        "retificacoes": RetAposentadoria("","regex"),
        "reversoes": Revertions("","regex"),
        "substituicao": Substituicao("","regex"),
        "sem_efeito_aposentadoria": SemEfeitoAposentadoria("","regex"),
        "cessoes": Cessoes("","regex")
    }
    assert len(objs) == 11
    assert len(objs) == len(needed)

def test_polished_core_act_df():
    assert type(ActsExtractor.get_act_df("aposentadoria", "", "regex")) == type(Retirements("", "regex").data_frame)
    assert type(ActsExtractor.get_act_df("reversoes", "", "regex")) == type(Revertions("", "regex").data_frame)
    assert type(ActsExtractor.get_act_df("nomeacao", "", "regex")) == type(NomeacaoComissionados("", "regex").data_frame)
    assert type(ActsExtractor.get_act_df("abono", "", "regex")) == type(AbonoPermanencia("", "regex").data_frame)
    assert type(ActsExtractor.get_act_df("retificacoes", "", "regex")) == type(RetAposentadoria("", "regex").data_frame)
    assert type(ActsExtractor.get_act_df("substituicao", "", "regex")) == type(Substituicao("", "regex").data_frame)
    assert type(ActsExtractor.get_act_df("efetivos_nome", "", "regex")) == type(NomeacaoEfetivos("", "regex").data_frame)
    assert type(ActsExtractor.get_act_df("efetivos_exo", "", "regex")) == type(ExoneracaoEfetivos("", "regex").data_frame)
    assert type(ActsExtractor.get_act_df("exoneracao", "", "regex")) == type(Exoneracao("", "regex").data_frame)
    assert type(ActsExtractor.get_act_df("sem_efeito_aposentadoria", "", "regex")) == type(SemEfeitoAposentadoria("", "regex").data_frame)
    assert type(ActsExtractor.get_act_df("cessoes", "", "regex")) == type(Cessoes("", "regex").data_frame)
    assert len(_acts_ids) == 11


def test_polished_core_get_all_df():
    data_frames = ActsExtractor.get_all_df("", "regex")
    dataframes = {
        "abono": AbonoPermanencia("","regex").data_frame,
        "aposentadoria": Retirements("","regex").data_frame,
        "efetivos_exo": ExoneracaoEfetivos("","regex").data_frame,
        "efetivos_nome": NomeacaoEfetivos("","regex").data_frame,
        "exoneracao": Exoneracao("","regex").data_frame,
        "nomeacao": NomeacaoComissionados("","regex").data_frame,
        "retificacoes": RetAposentadoria("","regex").data_frame,
        "reversoes": Revertions("","regex").data_frame,
        "substituicao": Substituicao("","regex").data_frame,
        "sem_efeito_aposentadoria": SemEfeitoAposentadoria("","regex").data_frame,
        "cessoes": Cessoes("","regex").data_frame
    }
    assert len(data_frames) == 11
    assert len(data_frames) == len(dataframes)

def test_polished_get_xml():
    xml = ActsExtractor.get_xml(""+os.path.dirname(__file__)+"/support/DODF 001 01-01-2019 EDICAO ESPECIAL.pdf", "regex", 0)
    assert isinstance(xml, XMLFy)



import os
import unicodedata
import json
import shutil

from glob import glob
from dodfminer.extract.pure.core import ContentExtractor

def test_pure_extract_text_single():
    file = ""+os.path.dirname(__file__)+"/support/dodfminer_sf.pdf"
    generated_txt = "BRASILIA - DF, QUINTA-FEIRA, 2 DE JANEIRO DE 2020 SECAO I SUMARIO"
    ContentExtractor.extract_text(file, single=True)
    assert os.path.isfile(file.replace("pdf", "txt"))
    assert generated_txt in open(file.replace("pdf", "txt")).read()
    os.remove(file.replace("pdf", "txt"))

def test_pure_extract_text_return_text():
    file = ""+os.path.dirname(__file__)+"/support/dodfminer_sf.pdf"
    generated_txt = "BRASILIA - DF, QUINTA-FEIRA, 2 DE JANEIRO DE 2020 SECAO I SUMARIO"
    assert generated_txt in ContentExtractor.extract_text(file)

def test_pure_extract_text_return_list():
    file = ""+os.path.dirname(__file__)+"/support/dodfminer_sf.pdf"
    generated_txt = "BRASILIA - DF, QUINTA-FEIRA, 2 DE JANEIRO DE 2020 SECAO I SUMARIO"
    assert len(ContentExtractor.extract_text(file, block=True)) > 0

def test_pure_extract_text_single_return_list():
    file = ""+os.path.dirname(__file__)+"/support/dodfminer_sf.pdf"
    generated_txt = "BRASILIA - DF, QUINTA-FEIRA, 2 DE JANEIRO DE 2020 SECAO I SUMARIO"
    ContentExtractor.extract_text(file, single=True, block=True)
    assert os.path.isfile(file.replace("pdf", "json"))
    assert len(json.loads(open(file.replace('pdf', 'json')).read())) > 0
    os.remove(file.replace("pdf", "json"))

# def test_pure_extract_text_norm():
#     file = ""+os.path.dirname(__file__)+"/support/dodfminer_sf.pdf"
#     generated_txt = "BRASILIA - DF, QUINTA-FEIRA, 2 DE JANEIRO DE 2020 SECAO I SUMARIO"
#     assert unicodedata.is_normalized("NFC", ContentExtractor.extract_text(file, norm="NFC"))

def test_pure_extract_structure_single():
    file = ""+os.path.dirname(__file__)+"/support/dodfminer_sf.pdf"
    ContentExtractor.extract_structure(file, single=True)
    assert os.path.isfile(file.replace("pdf", "json"))
    assert 'PODER EXECUTIVO' in json.loads(open(file.replace("pdf", "json")).read()).keys()
    os.remove(file.replace("pdf", "json"))

def test_pure_extract_structure():
    file = ""+os.path.dirname(__file__)+"/support/dodfminer_sf.pdf"
    assert 'PODER EXECUTIVO' in ContentExtractor.extract_structure(file).keys()

def test_pure_extract_to_txt():
    folder = ""+os.path.dirname(__file__)+"/support/dodf_pdfs"
    res_folder = folder + '/results/txt/2020/01_Janeiro/'
    ContentExtractor.extract_to_txt(folder)
    assert os.path.isdir(res_folder)
    assert len(glob(res_folder+'*.txt')) > 1

def test_pure_extract_to_txt():
    folder = ""+os.path.dirname(__file__)+"/support/dodf_pdfs"
    res_folder = folder + '/results/txt/2020/01_Janeiro/'
    ContentExtractor.extract_to_txt(folder)
    assert os.path.isdir(res_folder)
    assert len(glob(res_folder+'*.txt')) > 1
    shutil.rmtree(folder + '/results/')

def test_pure_extract_to_json_without_titles():
    folder = ""+os.path.dirname(__file__)+"/support/dodf_pdfs"
    res_folder = folder + '/results/json/2020/01_Janeiro/'
    ContentExtractor.extract_to_json(folder)
    assert os.path.isdir(res_folder)
    assert len(glob(res_folder+'*.json')) > 1
    shutil.rmtree(folder + '/results/')

def test_pure_extract_to_json_with_titles():
    folder = ""+os.path.dirname(__file__)+"/support/dodf_pdfs"
    res_folder = folder + '/results/json/2020/01_Janeiro/'
    ContentExtractor.extract_to_json(folder, titles_with_boxes=True)
    assert os.path.isdir(res_folder)
    assert len(glob(res_folder+'*.json')) > 1
    shutil.rmtree(folder + '/results/')

def test_pure_extract_to_json_already_exists(capsys):
    folder = ""+os.path.dirname(__file__)+"/support/dodf_pdfs"
    res_folder = folder + '/results/json/2020/01_Janeiro/'
    ContentExtractor.extract_to_json(folder, titles_with_boxes=True)
    ContentExtractor.extract_to_json(folder, titles_with_boxes=True)
    captured = capsys.readouterr()
    assert "JSON already exists" in captured.out
    shutil.rmtree(folder + '/results/')

def test_pure_extract_to_txt_already_exists(capsys):
    folder = ""+os.path.dirname(__file__)+"/support/dodf_pdfs"
    res_folder = folder + '/results/txt/2020/01_Janeiro/'
    ContentExtractor.extract_to_txt(folder)
    ContentExtractor.extract_to_txt(folder)
    captured = capsys.readouterr()
    assert "TXT already exists" in captured.out
    shutil.rmtree(folder + '/results/')
